@@ -26,6 +26,7 @@ import net.lingala.zip4j.model.EndOfCentralDirectoryRecord;
 import net.lingala.zip4j.model.ExtraDataRecord;
 import net.lingala.zip4j.model.FileHeader;
 import net.lingala.zip4j.model.LocalFileHeader;
+import net.lingala.zip4j.model.Zip4jConfig;
 import net.lingala.zip4j.model.Zip64EndOfCentralDirectoryLocator;
 import net.lingala.zip4j.model.Zip64EndOfCentralDirectoryRecord;
 import net.lingala.zip4j.model.Zip64ExtendedInfo;
@@ -39,7 +40,6 @@ import net.lingala.zip4j.util.RawIO;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.RandomAccessFile;
-import java.math.BigInteger;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -47,8 +47,9 @@ import java.util.List;
 
 import static net.lingala.zip4j.headers.HeaderUtil.decodeStringWithCharset;
 import static net.lingala.zip4j.util.BitUtils.isBitSet;
-import static net.lingala.zip4j.util.InternalZipConstants.BUFF_SIZE;
 import static net.lingala.zip4j.util.InternalZipConstants.ENDHDR;
+import static net.lingala.zip4j.util.InternalZipConstants.MAX_COMMENT_SIZE;
+import static net.lingala.zip4j.util.InternalZipConstants.ZIP4J_DEFAULT_CHARSET;
 import static net.lingala.zip4j.util.InternalZipConstants.ZIP_64_NUMBER_OF_ENTRIES_LIMIT;
 import static net.lingala.zip4j.util.InternalZipConstants.ZIP_64_SIZE_LIMIT;
 import static net.lingala.zip4j.util.Zip4jUtil.readFully;
@@ -59,10 +60,10 @@ import static net.lingala.zip4j.util.Zip4jUtil.readFully;
 public class HeaderReader {
 
   private ZipModel zipModel;
-  private RawIO rawIO = new RawIO();
-  private byte[] intBuff = new byte[4];
+  private final RawIO rawIO = new RawIO();
+  private final byte[] intBuff = new byte[4];
 
-  public ZipModel readAllHeaders(RandomAccessFile zip4jRaf, Charset charset) throws IOException {
+  public ZipModel readAllHeaders(RandomAccessFile zip4jRaf, Zip4jConfig zip4jConfig) throws IOException {
 
     if (zip4jRaf.length() < ENDHDR) {
       throw new ZipException("Zip file size less than minimum expected zip file size. " +
@@ -72,10 +73,11 @@ public class HeaderReader {
     zipModel = new ZipModel();
 
     try {
-      zipModel.setEndOfCentralDirectoryRecord(readEndOfCentralDirectoryRecord(zip4jRaf, rawIO, charset));
-    } catch (ZipException e){
+      zipModel.setEndOfCentralDirectoryRecord(readEndOfCentralDirectoryRecord(zip4jRaf, rawIO, zip4jConfig));
+    } catch (ZipException e) {
       throw e;
     } catch (IOException e) {
+      e.printStackTrace();
       throw new ZipException("Zip headers not found. Probably not a zip file or a corrupted zip file", e);
     }
 
@@ -97,22 +99,16 @@ public class HeaderReader {
       }
     }
 
-    zipModel.setCentralDirectory(readCentralDirectory(zip4jRaf, rawIO, charset));
+    zipModel.setCentralDirectory(readCentralDirectory(zip4jRaf, rawIO, zip4jConfig.getCharset()));
 
     return zipModel;
   }
 
-  private EndOfCentralDirectoryRecord readEndOfCentralDirectoryRecord(RandomAccessFile zip4jRaf, RawIO rawIO, Charset charset)
-      throws IOException {
+  private EndOfCentralDirectoryRecord readEndOfCentralDirectoryRecord(RandomAccessFile zip4jRaf, RawIO rawIO,
+                                                                      Zip4jConfig zip4jConfig) throws IOException {
 
-    long offsetEndOfCentralDirectory = zip4jRaf.length() - ENDHDR;
-    seekInCurrentPart(zip4jRaf, offsetEndOfCentralDirectory);
-    int headerSignature = rawIO.readIntLittleEndian(zip4jRaf);
-
-    if (headerSignature != HeaderSignature.END_OF_CENTRAL_DIRECTORY.getValue()) {
-      offsetEndOfCentralDirectory = determineOffsetOfEndOfCentralDirectory(zip4jRaf);
-      zip4jRaf.seek(offsetEndOfCentralDirectory + 4); // 4 to ignore reading signature again
-    }
+    long offsetEndOfCentralDirectory = locateOffsetOfEndOfCentralDirectory(zip4jRaf);
+    seekInCurrentPart(zip4jRaf, offsetEndOfCentralDirectory + 4);
 
     EndOfCentralDirectoryRecord endOfCentralDirectoryRecord = new EndOfCentralDirectoryRecord();
     endOfCentralDirectoryRecord.setSignature(HeaderSignature.END_OF_CENTRAL_DIRECTORY);
@@ -128,7 +124,7 @@ public class HeaderReader {
     endOfCentralDirectoryRecord.setOffsetOfStartOfCentralDirectory(rawIO.readLongLittleEndian(intBuff, 0));
 
     int commentLength = rawIO.readShortLittleEndian(zip4jRaf);
-    endOfCentralDirectoryRecord.setComment(readZipComment(zip4jRaf, commentLength, charset));
+    endOfCentralDirectoryRecord.setComment(readZipComment(zip4jRaf, commentLength, zip4jConfig.getCharset()));
 
     zipModel.setSplitArchive(endOfCentralDirectoryRecord.getNumberOfThisDisk() > 0);
     return endOfCentralDirectoryRecord;
@@ -140,7 +136,7 @@ public class HeaderReader {
 
     long offSetStartCentralDir = HeaderUtil.getOffsetStartOfCentralDirectory(zipModel);
     long centralDirEntryCount = getNumberOfEntriesInCentralDirectory(zipModel);
-    
+
     zip4jRaf.seek(offSetStartCentralDir);
 
     byte[] shortBuff = new byte[2];
@@ -168,7 +164,6 @@ public class HeaderReader {
 
       zip4jRaf.readFully(intBuff);
       fileHeader.setCrc(rawIO.readLongLittleEndian(intBuff, 0));
-      fileHeader.setCrcRawData(intBuff);
 
       fileHeader.setCompressedSize(rawIO.readLongLittleEndian(zip4jRaf, 4));
       fileHeader.setUncompressedSize(rawIO.readLongLittleEndian(zip4jRaf, 4));
@@ -202,11 +197,11 @@ public class HeaderReader {
         }
 
         fileHeader.setFileName(fileName);
-        fileHeader.setDirectory(fileName.endsWith("/") || fileName.endsWith("\\"));
       } else {
         fileHeader.setFileName(null);
       }
 
+      fileHeader.setDirectory(isDirectory(fileHeader.getExternalFileAttributes(), fileHeader.getFileName()));
       readExtraDataRecords(zip4jRaf, fileHeader);
       readZip64ExtendedInfo(fileHeader, rawIO);
       readAesExtraDataRecord(fileHeader, rawIO);
@@ -406,7 +401,7 @@ public class HeaderReader {
     return zip64EndOfCentralDirectoryRecord;
   }
 
-  private void readZip64ExtendedInfo(FileHeader fileHeader, RawIO rawIO) throws ZipException {
+  private void readZip64ExtendedInfo(FileHeader fileHeader, RawIO rawIO)  {
     if (fileHeader.getExtraDataRecords() == null || fileHeader.getExtraDataRecords().size() <= 0) {
       return;
     }
@@ -527,6 +522,9 @@ public class HeaderReader {
 
     //signature
     int sig = rawIO.readIntLittleEndian(inputStream);
+    if (sig == HeaderSignature.TEMPORARY_SPANNING_MARKER.getValue()) {
+      sig = rawIO.readIntLittleEndian(inputStream);
+    }
     if (sig != HeaderSignature.LOCAL_FILE_HEADER.getValue()) {
       return null;
     }
@@ -548,7 +546,6 @@ public class HeaderReader {
 
     readFully(inputStream, intBuff);
     localFileHeader.setCrc(rawIO.readLongLittleEndian(intBuff, 0));
-    localFileHeader.setCrcRawData(intBuff.clone());
 
     localFileHeader.setCompressedSize(rawIO.readLongLittleEndian(inputStream, 4));
     localFileHeader.setUncompressedSize(rawIO.readLongLittleEndian(inputStream, 4));
@@ -561,14 +558,8 @@ public class HeaderReader {
     if (fileNameLength > 0) {
       byte[] fileNameBuf = new byte[fileNameLength];
       readFully(inputStream, fileNameBuf);
-      // Modified after user reported an issue http://www.lingala.net/zip4j/forum/index.php?topic=2.0
-//				String fileName = new String(fileNameBuf, "Cp850");
-//				String fileName = Zip4jUtil.getCp850EncodedString(fileNameBuf);
-      String fileName = decodeStringWithCharset(fileNameBuf, localFileHeader.isFileNameUTF8Encoded(), charset);
 
-      if (fileName == null) {
-        throw new ZipException("file name is null, cannot assign file name to local file header");
-      }
+      String fileName = decodeStringWithCharset(fileNameBuf, localFileHeader.isFileNameUTF8Encoded(), charset);
 
       if (fileName.contains(":" + System.getProperty("file.separator"))) {
         fileName = fileName.substring(fileName.indexOf(":" + System.getProperty("file.separator")) + 2);
@@ -589,7 +580,7 @@ public class HeaderReader {
       if (localFileHeader.getEncryptionMethod() == EncryptionMethod.AES) {
         //Do nothing
       } else {
-        if (BigInteger.valueOf(localFileHeader.getGeneralPurposeFlag()[0]).testBit(6)) {
+        if (isBitSet(localFileHeader.getGeneralPurposeFlag()[0], 6)) {
           localFileHeader.setEncryptionMethod(EncryptionMethod.ZIP_STANDARD_VARIANT_STRONG);
         } else {
           localFileHeader.setEncryptionMethod(EncryptionMethod.ZIP_STANDARD);
@@ -702,28 +693,32 @@ public class HeaderReader {
     return zipModel.getEndOfCentralDirectoryRecord().getTotalNumberOfEntriesInCentralDirectory();
   }
 
-  private long determineOffsetOfEndOfCentralDirectory(RandomAccessFile randomAccessFile) throws IOException {
-    byte[] buff = new byte[BUFF_SIZE];
-    long currentFilePointer = randomAccessFile.getFilePointer();
+  private long locateOffsetOfEndOfCentralDirectory(RandomAccessFile randomAccessFile) throws IOException {
+    long zipFileSize = randomAccessFile.length();
+    if (zipFileSize < ENDHDR) {
+      throw new ZipException("Zip file size less than size of zip headers. Probably not a zip file.");
+    }
 
-    do {
-      int toRead = currentFilePointer > BUFF_SIZE ? BUFF_SIZE : (int) currentFilePointer;
-      // read 4 bytes again to make sure that the header is not spilled over
-      long seekPosition = currentFilePointer - toRead + 4;
-      if (seekPosition == 4) {
-        seekPosition = 0;
+    seekInCurrentPart(randomAccessFile, zipFileSize - ENDHDR);
+    if (rawIO.readIntLittleEndian(randomAccessFile) == HeaderSignature.END_OF_CENTRAL_DIRECTORY.getValue()) {
+      return zipFileSize - ENDHDR;
+    }
+
+    return locateOffsetOfEndOfCentralDirectoryByReverseSeek(randomAccessFile);
+  }
+
+  private long locateOffsetOfEndOfCentralDirectoryByReverseSeek(RandomAccessFile randomAccessFile) throws IOException {
+    long currentFilePointer = randomAccessFile.length() - ENDHDR;
+    // reverse seek for a maximum of MAX_COMMENT_SIZE bytes
+    long numberOfBytesToRead = randomAccessFile.length() < MAX_COMMENT_SIZE ? randomAccessFile.length() : MAX_COMMENT_SIZE;
+
+    while (numberOfBytesToRead > 0 && currentFilePointer > 0){
+      seekInCurrentPart(randomAccessFile, --currentFilePointer);
+      if (rawIO.readIntLittleEndian(randomAccessFile) == HeaderSignature.END_OF_CENTRAL_DIRECTORY.getValue()) {
+        return currentFilePointer;
       }
-
-      seekInCurrentPart(randomAccessFile, seekPosition);
-      randomAccessFile.read(buff, 0, toRead);
-      currentFilePointer = seekPosition;
-
-      for (int i = 0; i < toRead - 3; i++) {
-        if (rawIO.readIntLittleEndian(buff, i) == HeaderSignature.END_OF_CENTRAL_DIRECTORY.getValue()) {
-          return currentFilePointer + i;
-        }
-      }
-    } while (currentFilePointer > 0);
+      numberOfBytesToRead--;
+    };
 
     throw new ZipException("Zip headers not found. Probably not a zip file");
   }
@@ -744,10 +739,23 @@ public class HeaderReader {
     try {
       byte[] commentBuf = new byte[commentLength];
       raf.readFully(commentBuf);
-      return new String(commentBuf, charset);
+      return decodeStringWithCharset(commentBuf, false, charset != null ? charset : ZIP4J_DEFAULT_CHARSET);
     } catch (IOException e) {
       // Ignore any exception and set comment to null if comment cannot be read
       return null;
     }
   }
+
+  public boolean isDirectory(byte[] externalFileAttributes, String fileName) {
+    // first check if DOS attributes are set (lower order bytes from external attributes). If yes, check if the 4th bit
+    // which represents a directory is set. If UNIX attributes are set (higher order two bytes), check for the 6th bit
+    // in 4th byte which  represents a directory flag.
+    if (externalFileAttributes[0] != 0 && isBitSet(externalFileAttributes[0], 4)) {
+      return true;
+    } else if (externalFileAttributes[3] != 0 && isBitSet(externalFileAttributes[3], 6))  {
+      return true;
+    }
+
+    return fileName != null && (fileName.endsWith("/") || fileName.endsWith("\\"));
+  }
 }
