@@ -16,7 +16,6 @@
 
 package net.lingala.zip4j.io.inputstream;
 
-import net.lingala.zip4j.crypto.AESDecrypter;
 import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.headers.HeaderReader;
 import net.lingala.zip4j.headers.HeaderSignature;
@@ -24,6 +23,7 @@ import net.lingala.zip4j.model.DataDescriptor;
 import net.lingala.zip4j.model.ExtraDataRecord;
 import net.lingala.zip4j.model.FileHeader;
 import net.lingala.zip4j.model.LocalFileHeader;
+import net.lingala.zip4j.model.Zip4jConfig;
 import net.lingala.zip4j.model.enums.AesVersion;
 import net.lingala.zip4j.model.enums.CompressionMethod;
 import net.lingala.zip4j.model.enums.EncryptionMethod;
@@ -35,8 +35,8 @@ import java.io.PushbackInputStream;
 import java.nio.charset.Charset;
 import java.util.List;
 import java.util.zip.CRC32;
-import java.util.zip.DataFormatException;
 
+import static net.lingala.zip4j.util.InternalZipConstants.MIN_BUFF_SIZE;
 import static net.lingala.zip4j.util.Zip4jUtil.getCompressionMethod;
 
 public class ZipInputStream extends InputStream {
@@ -49,10 +49,12 @@ public class ZipInputStream extends InputStream {
   private CRC32 crc32 = new CRC32();
   private byte[] endOfEntryBuffer;
   private boolean canSkipExtendedLocalFileHeader = false;
-  private Charset charset;
+  private Zip4jConfig zip4jConfig;
+  private boolean streamClosed = false;
+  private boolean entryEOFReached = false;
 
   public ZipInputStream(InputStream inputStream) {
-    this(inputStream, null, InternalZipConstants.CHARSET_UTF_8);
+    this(inputStream, null, (Charset) null);
   }
 
   public ZipInputStream(InputStream inputStream, Charset charset) {
@@ -60,17 +62,21 @@ public class ZipInputStream extends InputStream {
   }
 
   public ZipInputStream(InputStream inputStream, char[] password) {
-    this(inputStream, password, InternalZipConstants.CHARSET_UTF_8);
+    this(inputStream, password, (Charset) null);
   }
 
   public ZipInputStream(InputStream inputStream, char[] password, Charset charset) {
-    if(charset == null) {
-      charset = InternalZipConstants.CHARSET_UTF_8;
+    this(inputStream, password, new Zip4jConfig(charset, InternalZipConstants.BUFF_SIZE));
+  }
+
+  public ZipInputStream(InputStream inputStream, char[] password, Zip4jConfig zip4jConfig) {
+    if (zip4jConfig.getBufferSize() < InternalZipConstants.MIN_BUFF_SIZE) {
+      throw new IllegalArgumentException("Buffer size cannot be less than " + MIN_BUFF_SIZE + " bytes");
     }
 
-    this.inputStream = new PushbackInputStream(inputStream, InternalZipConstants.BUFF_SIZE);
+    this.inputStream = new PushbackInputStream(inputStream, zip4jConfig.getBufferSize());
     this.password = password;
-    this.charset = charset;
+    this.zip4jConfig = zip4jConfig;
   }
 
   public LocalFileHeader getNextEntry() throws IOException {
@@ -82,7 +88,7 @@ public class ZipInputStream extends InputStream {
       readUntilEndOfEntry();
     }
 
-    localFileHeader = headerReader.readLocalFileHeader(inputStream, charset);
+    localFileHeader = headerReader.readLocalFileHeader(inputStream, zip4jConfig.getCharset());
 
     if (localFileHeader == null) {
       return null;
@@ -95,12 +101,17 @@ public class ZipInputStream extends InputStream {
       localFileHeader.setCrc(fileHeader.getCrc());
       localFileHeader.setCompressedSize(fileHeader.getCompressedSize());
       localFileHeader.setUncompressedSize(fileHeader.getUncompressedSize());
+      // file header's directory flag is more reliable than local file header's directory flag as file header has
+      // additional external file attributes which has a directory flag defined. In local file header, the only way
+      // to determine if an entry is directory is to check if the file name has a trailing forward slash "/"
+      localFileHeader.setDirectory(fileHeader.isDirectory());
       canSkipExtendedLocalFileHeader = true;
     } else {
       canSkipExtendedLocalFileHeader = false;
     }
 
     this.decompressedInputStream = initializeEntryInputStream(localFileHeader);
+    this.entryEOFReached = false;
     return localFileHeader;
   }
 
@@ -123,6 +134,10 @@ public class ZipInputStream extends InputStream {
 
   @Override
   public int read(byte[] b, int off, int len) throws IOException {
+    if (streamClosed) {
+      throw new IOException("Stream closed");
+    }
+
     if (len < 0) {
       throw new IllegalArgumentException("Negative read length");
     }
@@ -137,6 +152,10 @@ public class ZipInputStream extends InputStream {
       return -1;
     }
 
+    if (localFileHeader.isDirectory()) {
+      return -1;
+    }
+
     try {
       int readLen = decompressedInputStream.read(b, off, len);
 
@@ -148,8 +167,7 @@ public class ZipInputStream extends InputStream {
 
       return readLen;
     } catch (IOException e) {
-      if (e.getCause() != null && e.getCause() instanceof DataFormatException
-          && isEncryptionMethodZipStandard(localFileHeader)) {
+      if (isEncryptionMethodZipStandard(localFileHeader)) {
         throw new ZipException(e.getMessage(), e.getCause(), ZipException.Type.WRONG_PASSWORD);
       }
 
@@ -159,13 +177,31 @@ public class ZipInputStream extends InputStream {
 
   @Override
   public void close() throws IOException {
+    if (streamClosed) {
+      return;
+    }
+
     if (decompressedInputStream != null) {
       decompressedInputStream.close();
     }
+    this.streamClosed = true;
   }
 
-  public int getAvailableBytesInPushBackInputStream() throws IOException {
-    return inputStream.available();
+  @Override
+  public int available() throws IOException {
+    assertStreamOpen();
+    return entryEOFReached ? 0 : 1;
+  }
+
+  /**
+   * Sets the password for the inputstream. This password will be used for any subsequent encrypted entries that will be
+   * read from this stream. If this method is called when an entry is being read, it has no effect on the read action
+   * of the current entry, and the password will take effect from any subsequent entry reads.
+   *
+   * @param password Password to be used for reading of entries from the zip input stream
+   */
+  public void setPassword(char[] password) {
+    this.password = password;
   }
 
   private void endOfCompressedDataReached() throws IOException {
@@ -179,6 +215,7 @@ public class ZipInputStream extends InputStream {
     readExtendedLocalFileHeaderIfPresent();
     verifyCrc();
     resetFields();
+    this.entryEOFReached = true;
   }
 
   private DecompressedInputStream initializeEntryInputStream(LocalFileHeader localFileHeader) throws IOException {
@@ -187,26 +224,28 @@ public class ZipInputStream extends InputStream {
     return initializeDecompressorForThisEntry(cipherInputStream, localFileHeader);
   }
 
-  private CipherInputStream initializeCipherInputStream(ZipEntryInputStream zipEntryInputStream, LocalFileHeader localFileHeader) throws IOException {
+  private CipherInputStream initializeCipherInputStream(ZipEntryInputStream zipEntryInputStream,
+                                                        LocalFileHeader localFileHeader) throws IOException {
     if (!localFileHeader.isEncrypted()) {
-      return new NoCipherInputStream(zipEntryInputStream, localFileHeader, password);
+      return new NoCipherInputStream(zipEntryInputStream, localFileHeader, password, zip4jConfig.getBufferSize());
     }
 
     if (localFileHeader.getEncryptionMethod() == EncryptionMethod.AES) {
-      return new AesCipherInputStream(zipEntryInputStream, localFileHeader, password);
+      return new AesCipherInputStream(zipEntryInputStream, localFileHeader, password, zip4jConfig.getBufferSize());
     } else if (localFileHeader.getEncryptionMethod() == EncryptionMethod.ZIP_STANDARD) {
-      return new ZipStandardCipherInputStream(zipEntryInputStream, localFileHeader, password);
+      return new ZipStandardCipherInputStream(zipEntryInputStream, localFileHeader, password, zip4jConfig.getBufferSize());
     } else {
       final String message = String.format("Entry [%s] Strong Encryption not supported", localFileHeader.getFileName());
       throw new ZipException(message, ZipException.Type.UNSUPPORTED_ENCRYPTION);
     }
   }
 
-  private DecompressedInputStream initializeDecompressorForThisEntry(CipherInputStream cipherInputStream, LocalFileHeader localFileHeader) {
+  private DecompressedInputStream initializeDecompressorForThisEntry(CipherInputStream cipherInputStream,
+                                                                     LocalFileHeader localFileHeader) {
     CompressionMethod compressionMethod = getCompressionMethod(localFileHeader);
 
     if (compressionMethod == CompressionMethod.DEFLATE) {
-      return new InflaterInputStream(cipherInputStream);
+      return new InflaterInputStream(cipherInputStream, zip4jConfig.getBufferSize());
     }
 
     return new StoreInputStream(cipherInputStream);
@@ -303,7 +342,8 @@ public class ZipInputStream extends InputStream {
   }
 
   private void readUntilEndOfEntry() throws IOException {
-    if (localFileHeader.isDirectory() || localFileHeader.getCompressedSize() == 0) {
+    if ((localFileHeader.isDirectory() || localFileHeader.getCompressedSize() == 0)
+        && !localFileHeader.isDataDescriptorExists()) {
       return;
     }
 
@@ -311,10 +351,18 @@ public class ZipInputStream extends InputStream {
       endOfEntryBuffer = new byte[512];
     }
 
+    //noinspection StatementWithEmptyBody
     while (read(endOfEntryBuffer) != -1);
+    this.entryEOFReached = true;
   }
 
   private boolean isEncryptionMethodZipStandard(LocalFileHeader localFileHeader) {
     return localFileHeader.isEncrypted() && EncryptionMethod.ZIP_STANDARD.equals(localFileHeader.getEncryptionMethod());
   }
+
+  private void assertStreamOpen() throws IOException {
+    if (streamClosed) {
+      throw new IOException("Stream closed");
+    }
+  }
 }
