@@ -6,6 +6,7 @@ import net.lingala.zip4j.headers.HeaderSignature;
 import net.lingala.zip4j.headers.HeaderWriter;
 import net.lingala.zip4j.model.FileHeader;
 import net.lingala.zip4j.model.LocalFileHeader;
+import net.lingala.zip4j.model.Zip4jConfig;
 import net.lingala.zip4j.model.ZipModel;
 import net.lingala.zip4j.model.ZipParameters;
 import net.lingala.zip4j.model.enums.AesVersion;
@@ -19,6 +20,9 @@ import java.io.OutputStream;
 import java.nio.charset.Charset;
 import java.util.zip.CRC32;
 
+import static net.lingala.zip4j.util.InternalZipConstants.BUFF_SIZE;
+import static net.lingala.zip4j.util.InternalZipConstants.MIN_BUFF_SIZE;
+
 public class ZipOutputStream extends OutputStream {
 
   private CountingOutputStream countingOutputStream;
@@ -32,11 +36,12 @@ public class ZipOutputStream extends OutputStream {
   private CRC32 crc32 = new CRC32();
   private RawIO rawIO = new RawIO();
   private long uncompressedSizeForThisEntry = 0;
-  private Charset charset;
+  private Zip4jConfig zip4jConfig;
   private boolean streamClosed;
+  private boolean entryClosed = true;
 
   public ZipOutputStream(OutputStream outputStream) throws IOException {
-    this(outputStream, null, InternalZipConstants.CHARSET_UTF_8);
+    this(outputStream, null, null);
   }
 
   public ZipOutputStream(OutputStream outputStream, Charset charset) throws IOException {
@@ -44,21 +49,22 @@ public class ZipOutputStream extends OutputStream {
   }
 
   public ZipOutputStream(OutputStream outputStream, char[] password) throws IOException {
-    this(outputStream, password, InternalZipConstants.CHARSET_UTF_8);
+    this(outputStream, password, null);
   }
 
   public ZipOutputStream(OutputStream outputStream, char[] password, Charset charset) throws IOException {
-    this(outputStream, password, charset, new ZipModel());
+    this(outputStream, password, new Zip4jConfig(charset, BUFF_SIZE), new ZipModel());
   }
 
-  public ZipOutputStream(OutputStream outputStream, char[] password, Charset charset, ZipModel zipModel) throws IOException {
-    if(charset == null) {
-      charset = InternalZipConstants.CHARSET_UTF_8;
+  public ZipOutputStream(OutputStream outputStream, char[] password, Zip4jConfig zip4jConfig,
+                         ZipModel zipModel) throws IOException {
+    if (zip4jConfig.getBufferSize() < InternalZipConstants.MIN_BUFF_SIZE) {
+      throw new IllegalArgumentException("Buffer size cannot be less than " + MIN_BUFF_SIZE + " bytes");
     }
 
     this.countingOutputStream = new CountingOutputStream(outputStream);
     this.password = password;
-    this.charset = charset;
+    this.zip4jConfig = zip4jConfig;
     this.zipModel = initializeZipModel(zipModel, countingOutputStream);
     this.streamClosed = false;
     writeSplitZipHeaderIfApplicable();
@@ -72,6 +78,7 @@ public class ZipOutputStream extends OutputStream {
     //because local header data should be written first and then the encryption header data
     //and below initialisation writes encryption header data
     compressedOutputStream = initializeCompressedOutputStream(zipParameters);
+    this.entryClosed = false;
   }
 
   public void write(int b) throws IOException {
@@ -111,13 +118,18 @@ public class ZipOutputStream extends OutputStream {
       headerWriter.writeExtendedLocalHeader(localFileHeader, countingOutputStream);
     }
     reset();
+    this.entryClosed = true;
     return fileHeader;
   }
 
   @Override
   public void close() throws IOException {
+    if (!this.entryClosed) {
+      closeEntry();
+    }
+
     zipModel.getEndOfCentralDirectoryRecord().setOffsetOfStartOfCentralDirectory(countingOutputStream.getNumberOfBytesWritten());
-    headerWriter.finalizeZipFile(zipModel, countingOutputStream, charset);
+    headerWriter.finalizeZipFile(zipModel, countingOutputStream, zip4jConfig.getCharset());
     countingOutputStream.close();
     this.streamClosed = true;
   }
@@ -148,11 +160,11 @@ public class ZipOutputStream extends OutputStream {
 
   private void initializeAndWriteFileHeader(ZipParameters zipParameters) throws IOException {
     fileHeader = fileHeaderFactory.generateFileHeader(zipParameters, countingOutputStream.isSplitZipFile(),
-        countingOutputStream.getCurrentSplitFileCounter(), charset, rawIO);
+        countingOutputStream.getCurrentSplitFileCounter(), zip4jConfig.getCharset(), rawIO);
     fileHeader.setOffsetLocalHeader(countingOutputStream.getOffsetForNextEntry());
 
     localFileHeader = fileHeaderFactory.generateLocalFileHeader(fileHeader);
-    headerWriter.writeLocalFileHeader(zipModel, localFileHeader, countingOutputStream, charset);
+    headerWriter.writeLocalFileHeader(zipModel, localFileHeader, countingOutputStream, zip4jConfig.getCharset());
   }
 
   private void reset() throws IOException {
@@ -189,6 +201,8 @@ public class ZipOutputStream extends OutputStream {
       return new AesCipherOutputStream(zipEntryOutputStream, zipParameters, password);
     } else if (zipParameters.getEncryptionMethod() == EncryptionMethod.ZIP_STANDARD) {
       return new ZipStandardCipherOutputStream(zipEntryOutputStream, zipParameters, password);
+    } else if (zipParameters.getEncryptionMethod() == EncryptionMethod.ZIP_STANDARD_VARIANT_STRONG) {
+      throw new ZipException(EncryptionMethod.ZIP_STANDARD_VARIANT_STRONG + " encryption method is not supported");
     } else {
       throw new ZipException("Invalid encryption method");
     }
@@ -197,7 +211,7 @@ public class ZipOutputStream extends OutputStream {
   private CompressedOutputStream initializeCompressedOutputStream(CipherOutputStream cipherOutputStream,
                                                                   ZipParameters zipParameters) {
     if (zipParameters.getCompressionMethod() == CompressionMethod.DEFLATE) {
-      return new DeflaterOutputStream(cipherOutputStream, zipParameters.getCompressionLevel());
+      return new DeflaterOutputStream(cipherOutputStream, zipParameters.getCompressionLevel(), zip4jConfig.getBufferSize());
     }
 
     return new StoreOutputStream(cipherOutputStream);
