@@ -3,6 +3,7 @@ package net.lingala.zip4j;
 import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.io.inputstream.ZipInputStream;
 import net.lingala.zip4j.model.FileHeader;
+import net.lingala.zip4j.model.UnzipParameters;
 import net.lingala.zip4j.model.ZipParameters;
 import net.lingala.zip4j.model.enums.AesKeyStrength;
 import net.lingala.zip4j.model.enums.CompressionMethod;
@@ -20,11 +21,10 @@ import java.io.InputStream;
 import java.nio.file.Files;
 import java.nio.file.Paths;
 import java.util.ArrayList;
-import java.util.HashSet;
+import java.util.Arrays;
 import java.util.List;
-import java.util.Optional;
-import java.util.Set;
 
+import static net.lingala.zip4j.testutils.TestUtils.getFileNamesOfFiles;
 import static net.lingala.zip4j.testutils.TestUtils.getTestFileFromResources;
 import static net.lingala.zip4j.testutils.ZipFileVerifier.verifyZipFileByExtractingAllFiles;
 import static org.assertj.core.api.Assertions.assertThat;
@@ -134,6 +134,25 @@ public class ExtractZipFileIT extends AbstractIT {
     verifyNumberOfFilesInOutputFolder(outputFolder, 3);
   }
 
+  @Test
+  public void testExtractAllSkipsSymlinksWhenSymlinkExtractionSetToFalse() throws IOException {
+    ZipFile zipFile = createZipFileWithASymlink(createSymlink());
+
+    zipFile.extractAll(outputFolder.getPath(), buildUnzipParameters(false));
+
+    ZipFileVerifier.verifyFolderContentsSameAsSourceFiles(outputFolder);
+    verifyNumberOfFilesInOutputFolder(outputFolder, 4);
+  }
+
+  @Test
+  public void testExtractAllExtractsSymlinksWhenSymlinkExtractionSetToTrue() throws IOException {
+    ZipFile zipFile = createZipFileWithASymlink(createSymlink());
+
+    zipFile.extractAll(outputFolder.getPath(), buildUnzipParameters(true));
+
+    verifyNumberOfFilesInOutputFolder(outputFolder, 5);
+  }
+
   @Test
   public void testExtractFileWithFileHeaderWithAes128() throws IOException {
     ZipParameters zipParameters = createZipParameters(EncryptionMethod.AES, AesKeyStrength.KEY_STRENGTH_128);
@@ -175,6 +194,55 @@ public class ExtractZipFileIT extends AbstractIT {
     ZipFileVerifier.verifyFileContent(getTestFileFromResources("sample_text_large.txt"), outputFile);
   }
 
+  @Test
+  public void testExtractFileWithFileHeaderDoesNotExtractSymlinkWhenSymlinkExtractionIsNotSet() throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    FileHeader fileHeader = zipFile.getFileHeader(symlink.getName());
+    zipFile.extractFile(fileHeader, outputFolder.getPath(), buildUnzipParameters(false));
+
+    assertThat(outputFolder.listFiles()).isNull();
+  }
+
+  @Test
+  public void testExtractFileWithFileHeaderExtractsSymlinkWhenSymlinkExtractionIsSet() throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    FileHeader fileHeader = zipFile.getFileHeader(symlink.getName());
+    zipFile.extractFile(fileHeader, outputFolder.getPath(), buildUnzipParameters(true));
+
+    verifyNumberOfFilesInOutputFolder(outputFolder, 1);
+    assertThat(outputFolder.listFiles()[0].getName()).isEqualTo(symlink.getName());
+  }
+
+  @Test
+  public void testExtractFileWithFileHeaderAndNewFileDoesNotExtractSymlinkWhenSymlinkExtractionIsNotSet()
+      throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    FileHeader fileHeader = zipFile.getFileHeader(symlink.getName());
+    zipFile.extractFile(fileHeader, outputFolder.getPath(), "newFileName", buildUnzipParameters(false));
+
+    assertThat(outputFolder.listFiles()).isNull();
+  }
+
+  @Test
+  public void testExtractFileWithFileHeaderAndNewFileNameExtractsSymlinkWhenSymlinkExtractionIsSet()
+      throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    FileHeader fileHeader = zipFile.getFileHeader(symlink.getName());
+    String newFileName = "newName.link";
+    zipFile.extractFile(fileHeader, outputFolder.getPath(), newFileName, buildUnzipParameters(true));
+
+    verifyNumberOfFilesInOutputFolder(outputFolder, 1);
+    assertThat(outputFolder.listFiles()[0].getName()).isEqualTo(newFileName);
+  }
+
   @Test
   public void testExtractFileWithFileNameThrowsExceptionWhenFileNotFound() throws ZipException {
     ZipFile zipFile = new ZipFile(generatedZipFile, PASSWORD);
@@ -216,6 +284,50 @@ public class ExtractZipFileIT extends AbstractIT {
     ZipFileVerifier.verifyFileContent(getTestFileFromResources("sample_directory/favicon.ico"), outputFile);
   }
 
+  @Test
+  public void testExtractFileWithFileNameDoesNotExtractSymlinkWhenSymlinkExtractionIsNotSet() throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    zipFile.extractFile(symlink.getName(), outputFolder.getPath(), buildUnzipParameters(false));
+
+    assertThat(outputFolder.listFiles()).isNull();
+  }
+
+  @Test
+  public void testExtractFileWithFileNameExtractsSymlinkWhenSymlinkExtractionIsSet() throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    zipFile.extractFile(symlink.getName(), outputFolder.getPath(), buildUnzipParameters(true));
+
+    verifyNumberOfFilesInOutputFolder(outputFolder, 1);
+    assertThat(outputFolder.listFiles()[0].getName()).isEqualTo(symlink.getName());
+  }
+
+  @Test
+  public void testExtractFileWithFileNameAndNewFileNameDoesNotExtractSymlinkWhenSymlinkExtractionIsNotSet()
+      throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    zipFile.extractFile(symlink.getName(), outputFolder.getPath(), "newFileName", buildUnzipParameters(false));
+
+    assertThat(outputFolder.listFiles()).isNull();
+  }
+
+  @Test
+  public void testExtractFileWithFileNameAndNewFileNameExtractsSymlinkWhenSymlinkExtractionIsSet() throws IOException {
+    File symlink = createSymlink();
+    ZipFile zipFile = createZipFileWithASymlink(symlink);
+
+    String newFileName = "newName.link";
+    zipFile.extractFile(symlink.getName(), outputFolder.getPath(), newFileName, buildUnzipParameters(true));
+
+    verifyNumberOfFilesInOutputFolder(outputFolder, 1);
+    assertThat(outputFolder.listFiles()[0].getName()).isEqualTo(newFileName);
+  }
+
   @Test
   public void testExtractFilesThrowsExceptionForWrongPasswordForAes() throws IOException {
     ZipParameters zipParameters = createZipParameters(EncryptionMethod.AES, AesKeyStrength.KEY_STRENGTH_256);
@@ -279,7 +391,7 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
 
     assertThat(zipFile.getFileHeaders()).hasSize(44);
-    assertThat(Files.walk(outputFolder.toPath()).filter(Files::isRegularFile)).hasSize(44);
+    assertThat(getRegularFilesFromFolder(outputFolder)).hasSize(44);
    }
 
   @Test
@@ -300,7 +412,7 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
 
     assertThat(zipFile.getFileHeaders()).hasSize(3);
-    assertThat(Files.walk(outputFolder.toPath()).filter(Files::isRegularFile)).hasSize(3);
+    assertThat(getRegularFilesFromFolder(outputFolder)).hasSize(3);
   }
 
   @Test
@@ -311,7 +423,7 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
 
     assertThat(zipFile.getFileHeaders()).hasSize(1);
-    assertThat(Files.walk(outputFolder.toPath()).filter(Files::isRegularFile)).hasSize(1);
+    assertThat(getRegularFilesFromFolder(outputFolder)).hasSize(1);
   }
 
   @Test
@@ -323,8 +435,8 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
 
     assertThat(zipFile.getFileHeaders()).hasSize(2);
-    Set<String> filenameSet = new HashSet<>();
-    Files.walk(outputFolder.toPath()).forEach(file -> filenameSet.add(file.getFileName().toString()));
+    @SuppressWarnings("ConstantConditions")
+    List<String> filenameSet = getFileNamesOfFiles(Arrays.asList(outputFolder.listFiles()));
     assertThat(filenameSet.contains(expectedFileName)).isTrue();
   }
 
@@ -391,7 +503,7 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
     assertThat(outputFolder.listFiles()).isEmpty();
   }
-  
+
   @Test
   public void testExtractZipFileCRCError() throws IOException {
     ZipFile zipFile = new ZipFile(getTestArchiveFromResources("archive_with_invalid_zip64_headers.zip"));
@@ -495,6 +607,40 @@ public class ExtractZipFileIT extends AbstractIT {
     zipFile.extractAll(outputFolder.getPath());
   }
 
+  @Test
+  public void testExtractAllWithCustomBufferSize() throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setIncludeRootFolder(false);
+    zipFile.addFolder(getTestFileFromResources(""), zipParameters);
+    zipFile.setBufferSize(32 * 1024);
+
+    zipFile.extractAll(outputFolder.getPath());
+
+    ZipFileVerifier.verifyFolderContentsSameAsSourceFiles(outputFolder);
+    verifyNumberOfFilesInOutputFolder(outputFolder, 10);
+  }
+
+  @Test
+  public void testExtractingZipStandardEncryptedZipOverLoopAlwaysThrowsWrongPasswordTypeException() throws IOException {
+    for (int i = 0; i < 1000; i++) {
+      if (generatedZipFile.exists() && !generatedZipFile.delete()) {
+        throw new RuntimeException("Could not delete test zip file");
+      }
+
+      ZipFile zipFile = new ZipFile(generatedZipFile, "password".toCharArray());
+      addFileToZip(zipFile, "sample.pdf", EncryptionMethod.ZIP_STANDARD, "password");
+
+      try {
+        zipFile = new ZipFile(generatedZipFile, "WRONG_password".toCharArray());
+        zipFile.extractAll(outputFolder.getPath());
+      } catch (ZipException e) {
+        assertThat(e).isNotNull();
+        assertThat(e.getType()).isEqualTo(ZipException.Type.WRONG_PASSWORD);
+      }
+    }
+  }
+
   private void addFileToZip(ZipFile zipFile, String fileName, EncryptionMethod encryptionMethod, String password) throws ZipException {
     ZipParameters zipParameters = new ZipParameters();
     zipParameters.setEncryptFiles(encryptionMethod != null);
@@ -565,9 +711,13 @@ public class ExtractZipFileIT extends AbstractIT {
 
   private File getFileWithNameFrom(File outputFolder, String fileName) throws ZipException {
     List<File> filesInFolder = FileUtils.getFilesInDirectoryRecursive(outputFolder, true, true);
-    Optional<File> file = filesInFolder.stream().filter(e -> e.getName().equals(fileName)).findFirst();
-    assertThat(file).isPresent();
-    return file.get();
+    for (File file : filesInFolder) {
+      if (file.getName().equals(fileName)) {
+        return file;
+      }
+    }
+    fail("Could not find a file by name " + fileName);
+    return null;
   }
 
   private File createZipFileAndSplit(List<File> filesToAddToZip, long splitLength, boolean encrypt,
@@ -586,4 +736,36 @@ public class ExtractZipFileIT extends AbstractIT {
     ZipFile zipFile = new ZipFile(fileToExtract);
     zipFile.extractAll(outputFolder.getPath());
   }
+
+  private ZipFile createZipFileWithASymlink(File symlink) throws IOException {
+    ZipFile zipFile = new ZipFile(generatedZipFile);
+    ZipParameters zipParameters = new ZipParameters();
+    zipParameters.setSymbolicLinkAction(ZipParameters.SymbolicLinkAction.INCLUDE_LINK_AND_LINKED_FILE);
+    zipFile.addFiles(FILES_TO_ADD, zipParameters);
+    zipFile.addFile(symlink, zipParameters);
+    return zipFile;
+  }
+
+  private UnzipParameters buildUnzipParameters(boolean extractSymlinks) {
+    UnzipParameters unzipParameters = new UnzipParameters();
+    unzipParameters.setExtractSymbolicLinks(extractSymlinks);
+    return unzipParameters;
+  }
+
+  private File createSymlink() throws IOException {
+    File targetFile = getTestFileFromResources("file_PDF_1MB.pdf");
+    return TestUtils.createSymlink(targetFile, temporaryFolder.getRoot());
+  }
+
+  private List<File> getRegularFilesFromFolder(File folder) throws ZipException {
+    List<File> filesInFolder = FileUtils.getFilesInDirectoryRecursive(folder, false, false);
+
+    List<File> regularFiles = new ArrayList<>();
+    for (File file : filesInFolder) {
+      if (Files.isRegularFile(file.toPath())) {
+        regularFiles.add(file);
+      }
+    }
+    return regularFiles;
+  }
 }
