@@ -14,6 +14,7 @@ import java.io.File;
 import java.io.IOException;
 import java.util.Collections;
 
+import static net.lingala.zip4j.util.InternalZipConstants.MIN_BUFF_SIZE;
 import static org.assertj.core.api.Assertions.assertThat;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.reset;
@@ -34,6 +35,22 @@ public class ZipFileTest {
     zipFile = new ZipFile(sourceZipFile);
   }
 
+  @Test
+  public void testZipFileConstructorThrowsIllegalArgumentExceptionWhenFileParameterIsNull() {
+    expectedException.expect(IllegalArgumentException.class);
+    expectedException.expectMessage("input zip file parameter is null");
+
+    new ZipFile((File) null);
+  }
+
+  @Test
+  public void testZipFileConstructorWithPasswordThrowsIllegalArgumentExceptionWhenFileParameterIsNull() {
+    expectedException.expect(IllegalArgumentException.class);
+    expectedException.expectMessage("input zip file parameter is null");
+
+    new ZipFile((File) null, "password".toCharArray());
+  }
+
   @Test
   public void testCreateZipFileThrowsExceptionWhenZipFileExists() throws ZipException {
     reset(sourceZipFile);
@@ -43,7 +60,7 @@ public class ZipFileTest {
     expectedException.expectMessage("zip file: " + sourceZipFile + " already exists. " +
         "To add files to existing zip file use addFile method");
 
-    zipFile.createSplitZipFile(Collections.emptyList(), new ZipParameters(), true, 10000);
+    zipFile.createSplitZipFile(Collections.<File>emptyList(), new ZipParameters(), true, 10000);
   }
 
   @Test
@@ -59,7 +76,7 @@ public class ZipFileTest {
     expectedException.expect(ZipException.class);
     expectedException.expectMessage("input file List is null, cannot create zip file");
 
-    zipFile.createSplitZipFile(Collections.emptyList(), new ZipParameters(), true, 10000);
+    zipFile.createSplitZipFile(Collections.<File>emptyList(), new ZipParameters(), true, 10000);
   }
 
   @Test
@@ -138,6 +155,7 @@ public class ZipFileTest {
   @Test
   public void testAddFileAsFileThrowsExceptionWhenProgressMonitorStateIsBusy() throws ZipException {
     File fileToAdd = mockFile(true);
+    zipFile.setRunInThread(true);
     zipFile.getProgressMonitor().setState(ProgressMonitor.State.BUSY);
 
     expectedException.expect(ZipException.class);
@@ -159,12 +177,13 @@ public class ZipFileTest {
     expectedException.expect(ZipException.class);
     expectedException.expectMessage("input file List is null or empty");
 
-    zipFile.addFiles(Collections.emptyList());
+    zipFile.addFiles(Collections.<File>emptyList());
   }
 
   @Test
   public void testAddFilesThrowsExceptionWhenProgressMonitorStateIsBusy() throws ZipException {
     zipFile.getProgressMonitor().setState(ProgressMonitor.State.BUSY);
+    zipFile.setRunInThread(true);
 
     expectedException.expect(ZipException.class);
     expectedException.expectMessage("invalid operation - Zip4j is in busy state");
@@ -185,7 +204,7 @@ public class ZipFileTest {
     expectedException.expect(ZipException.class);
     expectedException.expectMessage("input file List is null or empty");
 
-    zipFile.addFiles(Collections.emptyList(), new ZipParameters());
+    zipFile.addFiles(Collections.<File>emptyList(), new ZipParameters());
   }
 
   @Test
@@ -199,6 +218,7 @@ public class ZipFileTest {
 
   @Test
   public void testAddFilesWithParametersThrowsExceptionWhenProgressMonitorStateIsBusy() throws ZipException {
+    zipFile.setRunInThread(true);
     zipFile.getProgressMonitor().setState(ProgressMonitor.State.BUSY);
 
     expectedException.expect(ZipException.class);
@@ -301,6 +321,18 @@ public class ZipFileTest {
     zipFile.addFolder(folderToAdd, null);
   }
 
+  @Test
+  public void testAddFolderThrowsExceptionWhenProgressMonitorStateIsBusy() throws ZipException {
+    File folderToAdd = mockFolder();
+    zipFile.setRunInThread(true);
+    zipFile.getProgressMonitor().setState(ProgressMonitor.State.BUSY);
+
+    expectedException.expect(ZipException.class);
+    expectedException.expectMessage("invalid operation - Zip4j is in busy state");
+
+    zipFile.addFolder(folderToAdd, new ZipParameters());
+  }
+
   @Test
   public void testAddStreamThrowsExceptionWhenInputStreamIsNull() throws ZipException {
     expectedException.expectMessage("inputstream is null, cannot add file to zip");
@@ -359,6 +391,7 @@ public class ZipFileTest {
 
   @Test
   public void testExtractFileWithFileHeaderWhenBusyStateThrowsException() throws ZipException {
+    zipFile.setRunInThread(true);
     zipFile.getProgressMonitor().setState(ProgressMonitor.State.BUSY);
 
     expectedException.expectMessage("invalid operation - Zip4j is in busy state");
@@ -593,6 +626,14 @@ public class ZipFileTest {
     assertThat(zipFile.toString()).isEqualTo("SOME_PATH");
   }
 
+  @Test
+  public void testSetBufferSizeThrowsExceptionWhenSizeLessThanExpected()  {
+    expectedException.expect(IllegalArgumentException.class);
+    expectedException.expectMessage("Buffer size cannot be less than " + MIN_BUFF_SIZE + " bytes");
+
+    zipFile.setBufferSize(MIN_BUFF_SIZE - 1);
+  }
+
   private File mockFile(boolean fileExists) {
     File file = mock(File.class);
     when(file.exists()).thenReturn(fileExists);
@@ -600,4 +641,12 @@ public class ZipFileTest {
     return file;
   }
 
-}
\ No newline at end of file
+  private File mockFolder() {
+    File folder = mock(File.class);
+    when(folder.exists()).thenReturn(true);
+    when(folder.toString()).thenReturn("SOME_PATH");
+    when(folder.isDirectory()).thenReturn(true);
+    when(folder.canRead()).thenReturn(true);
+    return folder;
+  }
+}
