@@ -4,6 +4,7 @@ import net.lingala.zip4j.AbstractIT;
 import net.lingala.zip4j.ZipFile;
 import net.lingala.zip4j.exception.ZipException;
 import net.lingala.zip4j.model.FileHeader;
+import net.lingala.zip4j.model.Zip4jConfig;
 import net.lingala.zip4j.model.ZipParameters;
 import net.lingala.zip4j.model.enums.AesKeyStrength;
 import net.lingala.zip4j.model.enums.AesVersion;
@@ -17,11 +18,13 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.ExpectedException;
 
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
@@ -31,6 +34,8 @@ import static net.lingala.zip4j.testutils.TestUtils.getTestFileFromResources;
 import static net.lingala.zip4j.testutils.ZipFileVerifier.verifyZipFileByExtractingAllFiles;
 import static net.lingala.zip4j.util.FileUtils.isMac;
 import static net.lingala.zip4j.util.FileUtils.isUnix;
+import static net.lingala.zip4j.util.FileUtils.isWindows;
+import static net.lingala.zip4j.util.InternalZipConstants.MIN_BUFF_SIZE;
 import static org.assertj.core.api.Assertions.assertThat;
 
 public class ZipOutputStreamIT extends AbstractIT {
@@ -38,6 +43,17 @@ public class ZipOutputStreamIT extends AbstractIT {
   @Rule
   public ExpectedException expectedException = ExpectedException.none();
 
+  @Test
+  public void testConstructorThrowsExceptionWhenBufferSizeIsLessThanExpected() throws IOException {
+    OutputStream outputStream = new ByteArrayOutputStream();
+    Zip4jConfig zip4jConfig = new Zip4jConfig(null, InternalZipConstants.MIN_BUFF_SIZE - 1);
+
+    expectedException.expect(IllegalArgumentException.class);
+    expectedException.expectMessage("Buffer size cannot be less than " + MIN_BUFF_SIZE + " bytes");
+
+    new ZipOutputStream(outputStream, null, zip4jConfig, null);
+  }
+
   @Test
   public void testZipOutputStreamStoreWithoutEncryption() throws IOException {
     testZipOutputStream(CompressionMethod.STORE, false, null, null, null);
@@ -203,6 +219,25 @@ public class ZipOutputStreamIT extends AbstractIT {
     verifyDefaultFileAttributes();
   }
 
+  @Test
+  public void testCreatingZipWithoutClosingEntryManuallySuccessfullyClosesEntry() throws IOException {
+    try (ZipOutputStream zipOutputStream = new ZipOutputStream(new FileOutputStream(generatedZipFile))) {
+      File fileToAdd = getTestFileFromResources("file_PDF_1MB.pdf");
+      byte[] buffer = new byte[InternalZipConstants.BUFF_SIZE];
+      int readLen;
+      ZipParameters zipParameters = new ZipParameters();
+      zipParameters.setFileNameInZip(fileToAdd.getName());
+      zipOutputStream.putNextEntry(zipParameters);
+      try (InputStream inputStream = new FileInputStream(fileToAdd)) {
+        while ((readLen = inputStream.read(buffer)) != -1) {
+          zipOutputStream.write(buffer, 0, readLen);
+        }
+      }
+    }
+
+    verifyZipFileByExtractingAllFiles(generatedZipFile, outputFolder, 1);
+  }
+
   private void testZipOutputStream(CompressionMethod compressionMethod, boolean encrypt,
                                    EncryptionMethod encryptionMethod, AesKeyStrength aesKeyStrength,
                                    AesVersion aesVersion)
@@ -214,12 +249,14 @@ public class ZipOutputStreamIT extends AbstractIT {
                                    EncryptionMethod encryptionMethod, AesKeyStrength aesKeyStrength,
                                    AesVersion aesVersion, boolean setLastModifiedTime)
           throws IOException {
-    testZipOutputStream(compressionMethod, encrypt, encryptionMethod, aesKeyStrength, aesVersion, true, FILES_TO_ADD, InternalZipConstants.CHARSET_UTF_8);
+    testZipOutputStream(compressionMethod, encrypt, encryptionMethod, aesKeyStrength, aesVersion, setLastModifiedTime,
+        FILES_TO_ADD, InternalZipConstants.CHARSET_UTF_8);
   }
 
   private void testZipOutputStream(CompressionMethod compressionMethod, boolean encrypt,
                                    EncryptionMethod encryptionMethod, AesKeyStrength aesKeyStrength,
-                                   AesVersion aesVersion, boolean setLastModifiedTime, List<File> filesToAdd, Charset charset)
+                                   AesVersion aesVersion, boolean setLastModifiedTime, List<File> filesToAdd,
+                                   Charset charset)
       throws IOException {
 
     ZipParameters zipParameters = buildZipParameters(compressionMethod, encrypt, encryptionMethod, aesKeyStrength);
@@ -380,10 +417,11 @@ public class ZipOutputStreamIT extends AbstractIT {
         } else {
           assertThat(fileHeader.getExternalFileAttributes()).isEqualTo(FileUtils.DEFAULT_POSIX_FILE_ATTRIBUTES);
         }
+      } else if (isWindows() && fileHeader.isDirectory()) {
+        assertThat(BitUtils.isBitSet(fileHeader.getExternalFileAttributes()[0], 4)).isTrue();
       } else {
         assertThat(fileHeader.getExternalFileAttributes()).isEqualTo(emptyAttributes);
       }
-
     }
   }
 
@@ -396,4 +434,4 @@ public class ZipOutputStreamIT extends AbstractIT {
 
     return zipFile;
   }
-}
\ No newline at end of file
+}
