@@ -15,7 +15,9 @@ import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
 import java.net.URLDecoder;
 import java.nio.file.Files;
+import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.util.ArrayList;
 import java.util.List;
 import java.util.Random;
 
@@ -131,6 +133,24 @@ public class TestUtils {
     return outputFile;
   }
 
+  public static File createSymlink(File targetFile, File rootFolder) throws IOException {
+    Path link = Paths.get(rootFolder.getAbsolutePath(), "symlink.link");
+    Files.createSymbolicLink(link, targetFile.toPath());
+    return link.toFile();
+  }
+
+  public static List<String> getFileNamesOfFiles(List<File> files) {
+    List<String> fileNames = new ArrayList<>();
+    if (files.isEmpty()) {
+      return fileNames;
+    }
+
+    for (File file : files) {
+      fileNames.add(file.getName());
+    }
+    return fileNames;
+  }
+
   private static OutputStream startNext7ZipSplitStream(File sourceFile, File outputFolder, int index) throws IOException {
     File outputFile = getFileNameFor7ZipSplitIndex(sourceFile, outputFolder, index);
     return new FileOutputStream(outputFile);
